Package: tcltk2
Type: Package
Version: 1.6.1
Date: 2025-06-01
Title: Tcl/Tk Additions
Description: A series of additional Tcl commands and Tk widgets to supplement the tcltk package.
Authors@R: c(person("Philippe", "Grosjean", role = c("aut", "cre"),
  email = "phgrosjean@sciviews.org",
  comment = c(ORCID = "0000-0002-2694-9471")))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Depends: R (>= 4.2.0), tcltk
Suggests: utils, covr, knitr, rmarkdown
SystemRequirements: Tcl/Tk (>= 8.5), Tktable (>= 2.9, optional)
License: LGPL-3
URL: https://github.com/SciViews/tcltk2/
BugReports: https://github.com/SciViews/tcltk2/issues/
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
NeedsCompilation: no
Packaged: 2025-06-01 16:52:06 UTC; phgrosjean
Author: Philippe Grosjean [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2694-9471>)
Repository: CRAN
Date/Publication: 2025-06-01 17:30:01 UTC
Built: R 4.6.0; ; 2026-04-26 09:20:29 UTC; unix
