# Generated by roxygen2: do not edit by hand

S3method("config<-",tk2label)
S3method("config<-",tk2widget)
S3method("disabled<-",tk2widget)
S3method("label<-",tk2widget)
S3method("selection<-",tk2listbox)
S3method("selection<-",tk2widget)
S3method("tag<-",tk2widget)
S3method("tip<-",tk2widget)
S3method("value<-",tk2listbox)
S3method("value<-",tk2widget)
S3method("values<-",tk2listbox)
S3method("values<-",tk2widget)
S3method(config,tk2label)
S3method(config,tk2widget)
S3method(disabled,tk2widget)
S3method(label,tk2widget)
S3method(print,tclTask)
S3method(print,tk2cfglist)
S3method(print,tk2widget)
S3method(selection,tk2listbox)
S3method(selection,tk2widget)
S3method(size,tk2listbox)
S3method(size,tk2widget)
S3method(state,tk2widget)
S3method(tag,tk2widget)
S3method(tip,tk2widget)
S3method(value,tk2listbox)
S3method(value,tk2widget)
S3method(values,tk2listbox)
S3method(values,tk2widget)
S3method(visibleItem,tk2listbox)
S3method(visibleItem,tk2widget)
export("config<-")
export("disabled<-")
export("label<-")
export("selection<-")
export("tag<-")
export("tip<-")
export("value<-")
export("values<-")
export(config)
export(disabled)
export(getLanguage)
export(is.tk)
export(is.tk2widget)
export(is.ttk)
export(label)
export(makeTclNames)
export(selection)
export(setLanguage)
export(size)
export(state)
export(tag)
export(tclAfter)
export(tclAfterCancel)
export(tclAfterInfo)
export(tclFun)
export(tclGetValue)
export(tclSetValue)
export(tclTaskChange)
export(tclTaskDelete)
export(tclTaskGet)
export(tclTaskRun)
export(tclTaskSchedule)
export(tclVarExists)
export(tclVarFind)
export(tclVarName)
export(tclmc)
export(tclmclocale)
export(tclmcset)
export(tip)
export(tk2button)
export(tk2canvas)
export(tk2cfglist)
export(tk2checkbutton)
export(tk2chooseFont)
export(tk2column)
export(tk2combobox)
export(tk2configList)
export(tk2ctext)
export(tk2dataList)
export(tk2dde)
export(tk2dde.exec)
export(tk2dde.poke)
export(tk2dde.request)
export(tk2dde.services)
export(tk2edit)
export(tk2entry)
export(tk2font.get)
export(tk2font.set)
export(tk2font.setstyle)
export(tk2frame)
export(tk2ico.create)
export(tk2ico.destroy)
export(tk2ico.list)
export(tk2ico.load)
export(tk2ico.set)
export(tk2ico.setFromFile)
export(tk2ico.sizes)
export(tk2insert.multi)
export(tk2killtip)
export(tk2label)
export(tk2labelframe)
export(tk2list.delete)
export(tk2list.get)
export(tk2list.insert)
export(tk2list.set)
export(tk2list.size)
export(tk2listbox)
export(tk2mclistbox)
export(tk2menu)
export(tk2menubutton)
export(tk2message)
export(tk2notebook)
export(tk2notetab)
export(tk2notetab.select)
export(tk2notetab.text)
export(tk2notetraverse)
export(tk2panedwindow)
export(tk2progress)
export(tk2radiobutton)
export(tk2reg.broadcast)
export(tk2reg.delete)
export(tk2reg.deletekey)
export(tk2reg.get)
export(tk2reg.keys)
export(tk2reg.set)
export(tk2reg.setkey)
export(tk2reg.type)
export(tk2reg.values)
export(tk2scale)
export(tk2scrollbar)
export(tk2separator)
export(tk2spinbox)
export(tk2state.set)
export(tk2style)
export(tk2swaplist)
export(tk2table)
export(tk2tablelist)
export(tk2text)
export(tk2theme)
export(tk2theme.elements)
export(tk2theme.list)
export(tk2tip)
export(tk2tree)
export(tk2unicode_bind)
export(tk2unicode_config)
export(tk2unicode_select)
export(value)
export(values)
export(visibleItem)
importFrom(tcltk,"tclvalue<-")
importFrom(tcltk,.Tcl)
importFrom(tcltk,.Tcl.callback)
importFrom(tcltk,as.tclObj)
importFrom(tcltk,tcl)
importFrom(tcltk,tclArray)
importFrom(tcltk,tclObj)
importFrom(tcltk,tclRequire)
importFrom(tcltk,tclServiceMode)
importFrom(tcltk,tclVar)
importFrom(tcltk,tclvalue)
importFrom(tcltk,tkactivate)
importFrom(tcltk,tkadd)
importFrom(tcltk,tkbind)
importFrom(tcltk,tkbindtags)
importFrom(tcltk,tkbutton)
importFrom(tcltk,tkcget)
importFrom(tcltk,tkconfigure)
importFrom(tcltk,tkcurselection)
importFrom(tcltk,tkdelete)
importFrom(tcltk,tkdestroy)
importFrom(tcltk,tkfocus)
importFrom(tcltk,tkfont.actual)
importFrom(tcltk,tkfont.configure)
importFrom(tcltk,tkfont.families)
importFrom(tcltk,tkfont.measure)
importFrom(tcltk,tkfont.names)
importFrom(tcltk,tkget)
importFrom(tcltk,tkgrab.set)
importFrom(tcltk,tkgrid)
importFrom(tcltk,tkgrid.columnconfigure)
importFrom(tcltk,tkgrid.configure)
importFrom(tcltk,tkgrid.rowconfigure)
importFrom(tcltk,tkindex)
importFrom(tcltk,tkinsert)
importFrom(tcltk,tklabel)
importFrom(tcltk,tkmessageBox)
importFrom(tcltk,tkscrollbar)
importFrom(tcltk,tksee)
importFrom(tcltk,tkselect)
importFrom(tcltk,tkselection.clear)
importFrom(tcltk,tkselection.set)
importFrom(tcltk,tkset)
importFrom(tcltk,tksize)
importFrom(tcltk,tktag.configure)
importFrom(tcltk,tktoplevel)
importFrom(tcltk,tkwait.window)
importFrom(tcltk,tkwidget)
importFrom(tcltk,tkwm.iconbitmap)
importFrom(tcltk,tkwm.title)
importFrom(tcltk,tkxview)
importFrom(tcltk,tkyview)
