#' @details
#' The \{tcltk2\} package provides additions to the \{tcltk\} base package.
#'
#' Many Tk widgets are added. Also, features of the Tcl language that can be
#' useful also in R are made accessible through R functions.
#'
#' @keywords internal
"_PACKAGE"

#' @importFrom tcltk tcl .Tcl tclRequire tclObj as.tclObj tclVar tclArray tclvalue tclvalue<- tclServiceMode .Tcl.callback tktoplevel tkwidget tkfont.names tkfont.configure tkfont.families tkfont.measure tkfont.actual tkconfigure tkcget tkcurselection tkselection.clear tkselection.set tksee tkset tksize tkwm.title tkxview tkyview tkindex tkdestroy tkinsert tkactivate tkbutton tkgrid tklabel tkgrid.configure tkgrid.columnconfigure tkgrid.rowconfigure tktag.configure tkgrab.set tkfocus tkwait.window tkwm.iconbitmap tkbind tkadd tkmessageBox tkscrollbar tkdelete tkselect tkbindtags tkget
# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL
